@@ -203,13 +203,25 @@ def test_ki_enabled_out_of_context():
     assert not _core.currently_ki_protected()
 
 
+def test_ki_disabled_in_del():
+    def nestedfunction():
+        return _core.currently_ki_protected()
+
+    def __del__():
+        assert _core.currently_ki_protected()
+        assert nestedfunction()
+
+    __del__()
+    assert not nestedfunction()
+
+
 def test_ki_protection_works():
     async def sleeper(name, record):
         try:
             while True:
                 await _core.checkpoint()
         except _core.Cancelled:
-            record.add((name + " ok"))
+            record.add(name + " ok")
 
     async def raiser(name, record):
         try:
@@ -221,7 +233,7 @@ def test_ki_protection_works():
             print("raised!")
             # Make sure we aren't getting cancelled as well as siginted
             await _core.checkpoint()
-            record.add((name + " raise ok"))
+            record.add(name + " raise ok")
             raise
         else:
             print("didn't raise!")
@@ -232,7 +244,7 @@ def test_ki_protection_works():
                     lambda _: _core.Abort.SUCCEEDED
                 )
             except _core.Cancelled:
-                record.add((name + " cancel ok"))
+                record.add(name + " cancel ok")
 
     # simulated control-C during raiser, which is *unprotected*
     print("check 1")
