@@ -5,120 +5,26 @@ from itertools import count
 import attr
 import outcome
 
-from . import _core
-from ._sync import CapacityLimiter
+import trio
 
-__all__ = [
-    "run_sync_in_worker_thread",
-    "current_default_worker_thread_limiter",
-    "BlockingTrioPortal",
-]
+from ._sync import CapacityLimiter
+from ._core import enable_ki_protection, disable_ki_protection, RunVar, TrioToken
+
+# Global due to Threading API, thread local storage for trio token
+TOKEN_LOCAL = threading.local()
 
 
 class BlockingTrioPortal:
-    """A portal that synchronous threads can reach through to run code in the
-    Trio thread.
-
-    Most Trio functions can only be called from the Trio thread, which is
-    sometimes annoying. What if you really need to call a Trio function from a
-    worker thread? That's where :class:`BlockingTrioPortal` comes in: it's the
-    rare Trio object whose methods can – in fact, must! – be called from
-    another thread, and it allows you to call all those other functions.
-
-    There is one complication: it's possible for a single Python program to
-    contain multiple calls to :func:`trio.run`, either in sequence – like in a
-    test suite that calls :func:`trio.run` for each test – or simultaneously
-    in different threads. So how do you control which :func:`trio.run` your
-    portal opens into?
-
-    The answer is that each :class:`BlockingTrioPortal` object is associated
-    with one *specific* call to :func:`trio.run`.
-
-    The simplest way to set this up is to instantiate the class with no
-    arguments inside Trio; this automatically binds it to the context where
-    you instantiate it::
-
-       async def some_function():
-           portal = trio.BlockingTrioPortal()
-           await trio.run_sync_in_worker_thread(sync_fn, portal)
-
-    Alternatively, you can pass an explicit :class:`trio.hazmat.TrioToken` to
-    specify the :func:`trio.run` that you want your portal to connect to.
-
-    """
-
     def __init__(self, trio_token=None):
         if trio_token is None:
-            trio_token = _core.current_trio_token()
+            trio_token = trio.hazmat.current_trio_token()
         self._trio_token = trio_token
 
-    # This is the part that runs in the trio thread
-    def _run_cb(self, q, afn, args):
-        @_core.disable_ki_protection
-        async def unprotected_afn():
-            return await afn(*args)
-
-        async def await_in_trio_thread_task():
-            q.put_nowait(await outcome.acapture(unprotected_afn))
-
-        _core.spawn_system_task(await_in_trio_thread_task, name=afn)
-
-    # This is the part that runs in the trio thread
-    def _run_sync_cb(self, q, fn, args):
-        @_core.disable_ki_protection
-        def unprotected_fn():
-            return fn(*args)
-
-        res = outcome.capture(unprotected_fn)
-        q.put_nowait(res)
-
-    def _do_it(self, cb, fn, *args):
-        try:
-            _core.current_task()
-        except RuntimeError:
-            pass
-        else:
-            raise RuntimeError(
-                "this is a blocking function; call it from a thread"
-            )
-        q = stdlib_queue.Queue()
-        self._trio_token.run_sync_soon(cb, q, fn, args)
-        return q.get().unwrap()
-
     def run(self, afn, *args):
-        """Run the given async function in the trio thread, blocking until it
-        is complete.
-
-        Returns or raises whatever the given function returns or raises. It
-        can also raise exceptions of its own:
-
-        Raises:
-          RunFinishedError: if the corresponding call to :func:`trio.run` has
-              already completed.
-          Cancelled: if the corresponding call to :func:`trio.run` completes
-              while ``afn(*args)`` is running, then ``afn`` is likely to raise
-              :class:`Cancelled`, and this will propagate out into
-          RuntimeError: if you try calling this from inside the Trio thread,
-              which would otherwise cause a deadlock.
-
-        """
-        return self._do_it(self._run_cb, afn, *args)
+        return from_thread_run(afn, *args, trio_token=self._trio_token)
 
     def run_sync(self, fn, *args):
-        """Run the given synchronous function in the trio thread, blocking
-        until it is complete.
-
-        Returns or raises whatever the given function returns or raises. It
-        can also exceptions of its own:
-
-        Raises:
-          RunFinishedError: if the corresponding call to :func:`trio.run` has
-              already completed.
-          RuntimeError: if you try calling this from inside the Trio thread,
-              which would otherwise cause a deadlock.
-
-        """
-        return self._do_it(self._run_sync_cb, fn, *args)
+        return from_thread_run_sync(fn, *args, trio_token=self._trio_token)
 
 
 ################################################################
@@ -221,19 +127,19 @@ class BlockingTrioPortal:
 # really the only real limit is on stack size actually *used*; how much you
 # *allocate* should be pretty much irrelevant.)
 
-_limiter_local = _core.RunVar("limiter")
+_limiter_local = RunVar("limiter")
 # I pulled this number out of the air; it isn't based on anything. Probably we
 # should make some kind of measurements to pick a good value.
 DEFAULT_LIMIT = 40
-_worker_thread_counter = count()
+_thread_counter = count()
 
 
-def current_default_worker_thread_limiter():
-    """Get the default :class:`CapacityLimiter` used by
-    :func:`run_sync_in_worker_thread`.
+def current_default_thread_limiter():
+    """Get the default `~trio.CapacityLimiter` used by
+    `trio.to_thread.run_sync`.
 
     The most common reason to call this would be if you want to modify its
-    :attr:`~CapacityLimiter.total_tokens` attribute.
+    :attr:`~trio.CapacityLimiter.total_tokens` attribute.
 
     """
     try:
@@ -253,104 +159,98 @@ class ThreadPlaceholder:
     name = attr.ib()
 
 
-@_core.enable_ki_protection
-async def run_sync_in_worker_thread(
-    sync_fn, *args, cancellable=False, limiter=None
-):
+@enable_ki_protection
+async def to_thread_run_sync(sync_fn, *args, cancellable=False, limiter=None):
     """Convert a blocking operation into an async operation using a thread.
 
     These two lines are equivalent::
 
         sync_fn(*args)
-        await run_sync_in_worker_thread(sync_fn, *args)
+        await trio.to_thread.run_sync(sync_fn, *args)
 
     except that if ``sync_fn`` takes a long time, then the first line will
     block the Trio loop while it runs, while the second line allows other Trio
     tasks to continue working while ``sync_fn`` runs. This is accomplished by
     pushing the call to ``sync_fn(*args)`` off into a worker thread.
 
+    From inside the worker thread, you can get back into Trio using the
+    functions in `trio.from_thread`.
+
     Args:
       sync_fn: An arbitrary synchronous callable.
       *args: Positional arguments to pass to sync_fn. If you need keyword
           arguments, use :func:`functools.partial`.
       cancellable (bool): Whether to allow cancellation of this operation. See
           discussion below.
-      limiter (None, CapacityLimiter, or CapacityLimiter-like object):
+      limiter (None, or CapacityLimiter-like object):
           An object used to limit the number of simultaneous threads. Most
-          commonly this will be a :class:`CapacityLimiter`, but it could be
+          commonly this will be a `~trio.CapacityLimiter`, but it could be
           anything providing compatible
           :meth:`~trio.CapacityLimiter.acquire_on_behalf_of` and
-          :meth:`~trio.CapacityLimiter.release_on_behalf_of`
-          methods. :func:`run_sync_in_worker_thread` will call
-          ``acquire_on_behalf_of`` before starting the thread, and
-          ``release_on_behalf_of`` after the thread has finished.
+          :meth:`~trio.CapacityLimiter.release_on_behalf_of` methods. This
+          function will call ``acquire_on_behalf_of`` before starting the
+          thread, and ``release_on_behalf_of`` after the thread has finished.
 
-          If None (the default), uses the default :class:`CapacityLimiter`, as
-          returned by :func:`current_default_worker_thread_limiter`.
+          If None (the default), uses the default `~trio.CapacityLimiter`, as
+          returned by :func:`current_default_thread_limiter`.
 
     **Cancellation handling**: Cancellation is a tricky issue here, because
     neither Python nor the operating systems it runs on provide any general
     mechanism for cancelling an arbitrary synchronous function running in a
-    thread. :func:`run_sync_in_worker_thread` will always check for
-    cancellation on entry, before starting the thread. But once the thread is
-    running, there are two ways it can handle being cancelled:
+    thread. This function will always check for cancellation on entry, before
+    starting the thread. But once the thread is running, there are two ways it
+    can handle being cancelled:
 
     * If ``cancellable=False``, the function ignores the cancellation and
       keeps going, just like if we had called ``sync_fn`` synchronously. This
       is the default behavior.
 
-    * If ``cancellable=True``, then ``run_sync_in_worker_thread`` immediately
-      raises :exc:`Cancelled`. In this case **the thread keeps running in
+    * If ``cancellable=True``, then this function immediately raises
+      `~trio.Cancelled`. In this case **the thread keeps running in
       background** – we just abandon it to do whatever it's going to do, and
       silently discard any return value or errors that it raises. Only use
       this if you know that the operation is safe and side-effect free. (For
-      example: :func:`trio.socket.getaddrinfo` is implemented using
-      :func:`run_sync_in_worker_thread`, and it sets ``cancellable=True``
-      because it doesn't really affect anything if a stray hostname lookup
-      keeps running in the background.)
+      example: :func:`trio.socket.getaddrinfo` is uses a thread with
+      ``cancellable=True``, because it doesn't really affect anything if a
+      stray hostname lookup keeps running in the background.)
 
       The ``limiter`` is only released after the thread has *actually*
-      finished – which in the case of cancellation may be some time after
-      :func:`run_sync_in_worker_thread` has returned. (This is why it's
-      crucial that :func:`run_sync_in_worker_thread` takes care of acquiring
-      and releasing the limiter.) If :func:`trio.run` finishes before the
-      thread does, then the limiter release method will never be called at
-      all.
+      finished – which in the case of cancellation may be some time after this
+      function has returned. If :func:`trio.run` finishes before the thread
+      does, then the limiter release method will never be called at all.
 
     .. warning::
 
-       You should not use :func:`run_sync_in_worker_thread` to call
-       long-running CPU-bound functions! In addition to the usual GIL-related
-       reasons why using threads for CPU-bound work is not very effective in
-       Python, there is an additional problem: on CPython, `CPU-bound threads
-       tend to "starve out" IO-bound threads
-       <https://bugs.python.org/issue7946>`__, so using
-       :func:`run_sync_in_worker_thread` for CPU-bound work is likely to
-       adversely affect the main thread running trio. If you need to do this,
-       you're better off using a worker process, or perhaps PyPy (which still
-       has a GIL, but may do a better job of fairly allocating CPU time
-       between threads).
+       You should not use this function to call long-running CPU-bound
+       functions! In addition to the usual GIL-related reasons why using
+       threads for CPU-bound work is not very effective in Python, there is an
+       additional problem: on CPython, `CPU-bound threads tend to "starve out"
+       IO-bound threads <https://bugs.python.org/issue7946>`__, so using
+       threads for CPU-bound work is likely to adversely affect the main
+       thread running Trio. If you need to do this, you're better off using a
+       worker process, or perhaps PyPy (which still has a GIL, but may do a
+       better job of fairly allocating CPU time between threads).
 
     Returns:
       Whatever ``sync_fn(*args)`` returns.
 
     Raises:
-      Whatever ``sync_fn(*args)`` raises.
+      Exception: Whatever ``sync_fn(*args)`` raises.
 
     """
-    await _core.checkpoint_if_cancelled()
-    token = _core.current_trio_token()
+    await trio.hazmat.checkpoint_if_cancelled()
+    token = trio.hazmat.current_trio_token()
     if limiter is None:
-        limiter = current_default_worker_thread_limiter()
+        limiter = current_default_thread_limiter()
 
     # Holds a reference to the task that's blocked in this function waiting
     # for the result – or None if this function was cancelled and we should
     # discard the result.
-    task_register = [_core.current_task()]
-    name = "trio-worker-{}".format(next(_worker_thread_counter))
+    task_register = [trio.hazmat.current_task()]
+    name = "trio-worker-{}".format(next(_thread_counter))
     placeholder = ThreadPlaceholder(name)
 
-    # This function gets scheduled into the trio run loop to deliver the
+    # This function gets scheduled into the Trio run loop to deliver the
     # thread's result.
     def report_back_in_trio_thread_fn(result):
         def do_release_then_return_result():
@@ -365,25 +265,35 @@ async def run_sync_in_worker_thread(
 
         result = outcome.capture(do_release_then_return_result)
         if task_register[0] is not None:
-            _core.reschedule(task_register[0], result)
+            trio.hazmat.reschedule(task_register[0], result)
 
     # This is the function that runs in the worker thread to do the actual
     # work and then schedule the call to report_back_in_trio_thread_fn
-    def worker_thread_fn():
-        result = outcome.capture(sync_fn, *args)
+    # Since this is spawned in a new thread, the trio token needs to be passed
+    # explicitly to it so it can inject it into thread local storage
+    def worker_thread_fn(trio_token):
+        TOKEN_LOCAL.token = trio_token
         try:
-            token.run_sync_soon(report_back_in_trio_thread_fn, result)
-        except _core.RunFinishedError:
-            # The entire run finished, so our particular task is certainly
-            # long gone -- it must have cancelled.
-            pass
+            result = outcome.capture(sync_fn, *args)
+            try:
+                token.run_sync_soon(report_back_in_trio_thread_fn, result)
+            except trio.RunFinishedError:
+                # The entire run finished, so our particular task is certainly
+                # long gone -- it must have cancelled.
+                pass
+        finally:
+            del TOKEN_LOCAL.token
 
     await limiter.acquire_on_behalf_of(placeholder)
     try:
         # daemon=True because it might get left behind if we cancel, and in
         # this case shouldn't block process exit.
+        current_trio_token = trio.hazmat.current_trio_token()
         thread = threading.Thread(
-            target=worker_thread_fn, name=name, daemon=True
+            target=worker_thread_fn,
+            args=(current_trio_token,),
+            name=name,
+            daemon=True
         )
         thread.start()
     except:
@@ -393,8 +303,134 @@ async def run_sync_in_worker_thread(
     def abort(_):
         if cancellable:
             task_register[0] = None
-            return _core.Abort.SUCCEEDED
+            return trio.hazmat.Abort.SUCCEEDED
         else:
-            return _core.Abort.FAILED
+            return trio.hazmat.Abort.FAILED
 
-    return await _core.wait_task_rescheduled(abort)
+    return await trio.hazmat.wait_task_rescheduled(abort)
+
+
+def _run_fn_as_system_task(cb, fn, *args, trio_token=None):
+    """Helper function for from_thread.run and from_thread.run_sync.
+
+    Since this internally uses TrioToken.run_sync_soon, all warnings about
+    raised exceptions canceling all tasks should be noted.
+    """
+
+    if trio_token and not isinstance(trio_token, TrioToken):
+        raise RuntimeError("Passed kwarg trio_token is not of type TrioToken")
+
+    if not trio_token:
+        try:
+            trio_token = TOKEN_LOCAL.token
+        except AttributeError:
+            raise RuntimeError(
+                "this thread wasn't created by Trio, pass kwarg trio_token=..."
+            )
+
+    # TODO: This is only necessary for compatibility with BlockingTrioPortal.
+    # once that is deprecated, this check should no longer be necessary because
+    # thread local storage (or the absence of) is sufficient to check if trio
+    # is running in a thread or not.
+    try:
+        trio.hazmat.current_task()
+    except RuntimeError:
+        pass
+    else:
+        raise RuntimeError(
+            "this is a blocking function; call it from a thread"
+        )
+
+    q = stdlib_queue.Queue()
+    trio_token.run_sync_soon(cb, q, fn, args)
+    return q.get().unwrap()
+
+
+def from_thread_run(afn, *args, trio_token=None):
+    """Run the given async function in the parent Trio thread, blocking until it
+    is complete.
+
+    Returns:
+      Whatever ``afn(*args)`` returns.
+
+    Returns or raises whatever the given function returns or raises. It
+    can also raise exceptions of its own:
+
+    Raises:
+        RunFinishedError: if the corresponding call to :func:`trio.run` has
+            already completed.
+        Cancelled: if the corresponding call to :func:`trio.run` completes
+            while ``afn(*args)`` is running, then ``afn`` is likely to raise
+            :exc:`trio.Cancelled`, and this will propagate out into
+        RuntimeError: if you try calling this from inside the Trio thread,
+            which would otherwise cause a deadlock.
+        AttributeError: if no ``trio_token`` was provided, and we can't infer
+            one from context.
+
+    **Locating a Trio Token**: There are two ways to specify which
+    `trio.run` loop to reenter:
+
+        - Spawn this thread from `trio.to_thread.run_sync`. Trio will
+          automatically capture the relevant Trio token and use it when you
+          want to re-enter Trio.
+        - Pass a keyword argument, ``trio_token`` specifiying a specific
+          `trio.run` loop to re-enter. This is useful in case you have a
+          "foreign" thread, spawned using some other framework, and still want
+          to enter Trio.
+    """
+
+    def callback(q, afn, args):
+        @disable_ki_protection
+        async def unprotected_afn():
+            return await afn(*args)
+
+        async def await_in_trio_thread_task():
+            q.put_nowait(await outcome.acapture(unprotected_afn))
+
+        trio.hazmat.spawn_system_task(await_in_trio_thread_task, name=afn)
+
+    return _run_fn_as_system_task(callback, afn, *args, trio_token=trio_token)
+
+
+def from_thread_run_sync(fn, *args, trio_token=None):
+    """Run the given sync function in the parent Trio thread, blocking until it
+    is complete.
+
+    Returns:
+      Whatever ``fn(*args)`` returns.
+
+    Returns or raises whatever the given function returns or raises. It
+    can also raise exceptions of its own:
+
+    Raises:
+        RunFinishedError: if the corresponding call to `trio.run` has
+            already completed.
+        Cancelled: if the corresponding call to `trio.run` completes
+            while ``afn(*args)`` is running, then ``afn`` is likely to raise
+            :exc:`trio.Cancelled`, and this will propagate out into
+        RuntimeError: if you try calling this from inside the Trio thread,
+            which would otherwise cause a deadlock.
+        AttributeError: if no ``trio_token`` was provided, and we can't infer
+            one from context.
+
+    **Locating a Trio Token**: There are two ways to specify which
+    `trio.run` loop to reenter:
+
+        - Spawn this thread from `trio.to_thread.run_sync`. Trio will
+          automatically capture the relevant Trio token and use it when you
+          want to re-enter Trio.
+        - Pass a keyword argument, ``trio_token`` specifiying a specific
+          `trio.run` loop to re-enter. This is useful in case you have a
+          "foreign" thread, spawned using some other framework, and still want
+          to enter Trio.
+    """
+
+    def callback(q, fn, args):
+        @disable_ki_protection
+        def unprotected_fn():
+            return fn(*args)
+
+        res = outcome.capture(unprotected_fn)
+        q.put_nowait(res)
+
+    return _run_fn_as_system_task(callback, fn, *args, trio_token=trio_token)
